import hou

from controlVariables import *
from uENode import *
from stdNetworkNodes import *

from voronoise import *
from grid import *
from radial import *
from compositing import *
from districts import *

from lots import *
from buildings import *

# ===========================================================
#                uEPatternNodeFactory
# ===========================================================
class uEPatternNodeFactory:
	def __init__(self):
		self.__createContainer()
	def getContainer(self):
		return self.geo
	def __createContainer(self):
		self.geo = hou.node('/obj').createNode('geo')
		controlVariables.geo = self.geo
		#delete the file node
		for n in self.geo.children():
			n.destroy()
	def createNode(self,nodeName):
		if (nodeName == 'majorRoads'): 
			outputNode = PatternMajorRoads()
		elif (nodeName == 'minorRoads'): 
			outputNode = DicedMinorRoads()
		elif (nodeName == 'blocks'): 
			outputNode = Blocks()
		elif (nodeName == 'lots'):
			outputNode = Lots()
		elif (nodeName == 'buildings'):
			outputNode = Buildings()
		elif (nodeName == 'finalAppearance'): 
			outputNode = FinalAppearance()
		elif (nodeName == 'floor'): 
			outputNode = Floor()
		elif (nodeName == 'filter'): 
			outputNode = ConstructionFilter()
		elif (nodeName == 'stats'):
			outputNode = Stats()
		outputNode.setContainer(self.geo)
		outputNode.construct()
		return outputNode
	def finishConstruction(self):
		self.geo.setDisplayFlag(True)
		for n in self.geo.children():
			n.moveToGoodPosition()			
		for n in hou.node('/obj').children():
			n.moveToGoodPosition()

# ===========================================================
#                PatternMajorRoads
# ===========================================================
class PatternMajorRoads(urbanEngineNode):
	def getNodeName(self):
		return 'PatternMajorRoads'
	def __vex(self, type):
		if type == 'surface':
			sur = hou.node('/vex').createNode('surface') #create surface
			sur.setName('CityPattern') #change its name
			controlVariables.sur = sur
		else:
			sur = hou.node('/vex').createNode('cop2filter')#create filter
			sur.setName('myPattern') #change its name
			controlVariables.sur = sur

	def __startPatternConstruction(self):
		if (controlVariables.districtMap != None):
			d = districts()
			self.__vex('surface')
			d.districts('surface')
			self.__vex('filter')
			d.districts('filter')
		elif (controlVariables.pattern == 'voronoi'):
			v = voronoise()
			self.__vex('surface')
			v.createSurface('surface')
			self.__vex('filter')
			v.createSurface('filter')
		elif (controlVariables.pattern == 'grid'): 
			g = grid()
			self.__vex('surface')
			g.createSurface('surface')
			self.__vex('filter')
			g.createSurface('filter')
		elif (controlVariables.pattern == 'radial'):
			r = radial()
			self.__vex('surface')
			r.createSurface('surface')
			self.__vex('filter')
			r.createSurface('filter')
		#compositing should be last, because it needs the self.__vex nodes to be already defined
		c = compositing()
		c.compositing()
					
		for n in hou.node('/vex').children():
			n.moveToGoodPosition()
	
	def __init__(self):
		urbanEngineNode.__init__(self, False)

	def __carregarMapaCiutatReal(self,tra):
		tra.parm('file').set(controlVariables.loadMap)#use the map
		tra.parm('usecop1').set(0)
		return tra

	def __carregarImatgeCOP2(self,tra):
		tra.parm('coppath').set('/img/img1/final')#use COP2
		tra.parm('usecop1').set(1)
		return tra

	def __esborrarPrimitivaExterior(self,tra):
		dele = self.geo.createNode('delete')
		dele.setFirstInput(tra)
		dele.parm('pattern').set('0')
		return dele

	def construct(self):
		self.__startPatternConstruction()
		tra = self.geo.createNode('trace')
		tra = self.__carregarImatgeCOP2(tra)
		tra.parm('thresh').set(controlVariables.traceThreshold)
		tra.parm('doresample').set(1)
		tra.parm('step').set(15)

		if (controlVariables.districtMap!=None) or (controlVariables.pattern == 'voronoi') or (controlVariables.needsDelete == 'yes'):
			tra = self.__esborrarPrimitivaExterior(tra)

		self.first = tra
		self.last = tra


