import sys
import hou

from controlVariables import *
from uENode import *

from PatternMajorRoads import *
from streets import *
from lots import *
from buildings import *


class uEPatternNodeFactory:
	def __init__(self):
		self.__createContainer()
	def getContainer(self):
		return self.geo
	def __createContainer(self):
		self.geo = hou.node('/obj').createNode('geo')
		controlVariables.geo = self.geo
		#delete the file node
		for n in self.geo.children():
			n.destroy()
	def createNode(self,nodeName):
		if (nodeName == 'majorRoads'): 
			outputNode = MajorRoads()
		elif (nodeName == 'minorRoads'): 
			outputNode = MinorRoads()
		elif (nodeName == 'blocks'): 
			outputNode = Blocks()
		elif (nodeName == 'lots'):
			outputNode = Lots()
		elif (nodeName == 'buildings'):
			outputNode = Buildings()
		elif (nodeName == 'finalAppearance'): 
			outputNode = FinalAppearance()
		elif (nodeName == 'floor'): 
			outputNode = Floor()
		elif (nodeName == 'filter'): 
			outputNode = ConstructionFilter()
		outputNode.setContainer(self.geo)
		outputNode.construct()
		return outputNode
	def finishConstruction(self):
		self.geo.setDisplayFlag(True)
		for n in self.geo.children():
			n.moveToGoodPosition()			
		for n in hou.node('/obj').children():
			n.moveToGoodPosition()

class MinorRoads(urbanEngineNode):
	def __init__(self):
		urbanEngineNode.__init__(self)
	def getNodeName(self):
		return 'MinorRoads'

	def construct(self):
		if (controlVariables.pattern == 'voronoi') or (controlVariables.pattern == 'grid') or (controlVariables.pattern == 'radial'):
			#measure primitives area
			mea3 = self.geo.createNode('measure') 
			mea3.parm('override').set(1) #override name
			mea3.parm('type').set(1) #compute the areas
			mea3.parm('attribname').set('areaLots')
			mea3.setFirstInput(self.first)

			attpr2 = self.geo.createNode('attribpromote')
			attpr2.setName('avgarea')
			attpr2.parm('inname').set('areaLots') #original name
			attpr2.parm('inclass').set(1) #original class: primitive
			attpr2.parm('outclass').set(0) #new class: detail
			attpr2.parm('method').set(2) #promotion method: average
			attpr2.parm('useoutname').set(1) #change new name
			attpr2.parm('outname').set('avgarea')
			attpr2.setFirstInput(mea3)

			s = streets()
			newStreets = s.streets(attpr2)

			self.last = newStreets
		else:
			pass

class Blocks(urbanEngineNode):
	def __init__(self):
		urbanEngineNode.__init__(self)
	def getNodeName(self):
		return 'Blocks'

	def construct(self):
		#measure primitives area
		mea = self.geo.createNode('measure')
		mea.parm('override').set(1) #override name
		mea.parm('type').set(1) #compute the areas
		mea.setFirstInput(self.first)

		attpr = self.geo.createNode('attribpromote')
		attpr.setName('areaMeasurer')
		attpr.parm('inname').set('area') #original name
		attpr.parm('inclass').set(1) #original class: primitive
		attpr.parm('outclass').set(0) #new class: detail
		attpr.parm('method').set(2) #promotion method: average
		attpr.parm('useoutname').set(1) #change new name
		attpr.parm('outname').set('totsize')
		attpr.setFirstInput(mea)

		#delete too small primitives
		del5 = self.geo.createNode('delete')
		del5.setName('del')
		del5.parm('groupop').set(2) #delete by expression
		if (controlVariables.districtMap!=None) or (controlVariables.pattern == 'grid'):
			del5.parm('filter').setExpression("$AREA < 0.7 * detail(\"../areaMeasurer\",\"totsize\",0)")
		else:
			del5.parm('filter').setExpression("$AREA < 0.2 * detail(\"../areaMeasurer\",\"totsize\",0)")
		del5.setFirstInput(mea)
		
		self.last = del5

class NullBlocks(urbanEngineNode):
	def __init__(self):
		urbanEngineNode.__init__(self)
	def getNodeName(self):
		return 'NullBlocks'

	def construct(self):
		#Simply do nothing. For this, we create a silly null node. 
		#Directly from the Houdini (null node) help:
		#     This SOP passes through its input untouched, so it 
		#     does no cooking. Adding a null does not increase memory 
		#     usage (except when 'Cache Input' is on).
		pass

class FinalAppearance(urbanEngineNode):
	def __init__(self):
		urbanEngineNode.__init__(self)
		self.i = 0
	def getNodeName(self):
		return 'FinalAppearance'

	def __scale(self, tra, name):
		#scale the whole terrain
		scale2 = self.geo.createNode('xform')
		scale2.setName(name)
		scale2.parm('sx').set(1)
		scale2.parm('sy').set(1)
		scale2.parm('sz').set(1)
		scale2.setFirstInput(tra.getLast())
		return scale2

	def construct(self):
		self.first.setDisplayFlag(True)

	def setFirstInput(self, node_to_become_input, output_index=0):
		scale = self.__scale(node_to_become_input,'finalScale'+str(self.i))
		self.i += 1
		self.first.setFirstInput(scale, output_index)

	def setNextInput(self, node_to_become_input, output_index=0):
		scale = self.__scale(node_to_become_input,'finalScale'+str(self.i))
		self.i += 1
		self.first.setNextInput(scale, output_index)

	def setInput(self, input_index, node_to_become_input, output_index=0):
		scale = self.__scale(node_to_become_input,'finalScale'+str(self.i))
		self.i += 1
		self.first.setInput(input_index, scale, output_index)


class Floor(urbanEngineNode):
	def __init__(self):
		urbanEngineNode.__init__(self)
		self.matchsize = None
	def getNodeName(self):
		return 'Floor'

	def construct(self):
		grid = self.geo.createNode('grid')
		grid.parm('orient').set(0) #xy pane
		grid.parm('sizex').set(1)
		grid.parm('sizex').set(1)

		shade = self.geo.createNode('uvquickshade')
		shade.setFirstInput(grid)
		shade.parm('texture').set(controlVariables.floor)
		shade.parm('texture_axis').set(2) #z axis

		temp = self.geo.createNode('null')
		temp.setName('floorSizeRef')
		
		self.matchsize = self.geo.createNode('matchsize')
		self.matchsize.setFirstInput(shade)
		self.matchsize.setInput(1,temp)

		xform = self.geo.createNode('xform')
		xform.setFirstInput(self.matchsize)
		xform.parm('tz').set(-0.04)

		temp.setFirstInput(self.first)
		self.last = xform

class ConstructionFilter(urbanEngineNode):
	def __init__(self):
		urbanEngineNode.__init__(self)
	def getNodeName(self):
		return 'ConstructionFilter'

	def construct(self):
		del7 = self.geo.createNode('delete')
		del7.parm('negate').set(1) #delete non-selected
		del7.parm('groupop').set(1) #delete by range
		del7.parm('select1').set(1)
		del7.parm('select2').set(1)
		del7.parm('rangestart').set(0)
		del7.parm('rangeend').set(0) #setExpression('$N')
		del7.setFirstInput(self.first)
		
		self.last = del7
