import sys
import hou

#from buildings import *
from uENode import *

from controlVariables import *

class Lots(urbanEngineNode):
	def __init__(self):
		urbanEngineNode.__init__(self)
		self.resampleNode = None
		self.detailExpr = ''
	def getNodeName(self):
		return 'Lots'
	def construct(self):
		entry = self.geo.createNode('null')
		entry.setName('entryLots')
		entry.setFirstInput(self.first)

		#delete all of the primitives but one at the time
		del6 = self.geo.createNode('delete')
		del6.parm('affectnumber').set(0)
		del6.parm('group').set(r'`stamp("../finalcopy","ccy",0)`')
		del6.parm('negate').set(1)
		del6.setFirstInput(entry)
		
		mea3 = self.geo.createNode('measure')
		mea3.setName('measurePerim')
		mea3.parm('override').set(1)
		mea3.parm('attribname').set('perim')
		mea3.parm('type').set('perimeter') #perimeter
		mea3.setFirstInput(del6)
	
		attp = self.crearPrimitivaInterior(mea3) #here yhe node point1 is created
		
		#we want to have for each segment a random amount of extrusion
		#extract segments
		car = self.geo.createNode('carve')
		car.parm('secondu').set(1)
		car.parm('domainu2').set(1)
		car.parm('domainu1').setExpression('3*stamp("../copySegments","seg",6)/npoints("../point1")')
		car.parm('domainu2').setExpression('3*(stamp("../copySegments","seg",6)+1)/npoints("../point1")')
		car.setFirstInput(attp)
		
		#move along the normal
		pea2 = self.geo.createNode('peak')
		pea2.parm('updatenmls').set(0) #don't recompute point normals
		pea2.parm('dist').setExpression('1.5*sqrt(rand(stamp("../copySegments","seg",0))) * prim("../distPAttriPromote",0,"distP",0) / point("../carve1",0,"dist",0)')
		#pea2.parm('dist').setExpression('1.2*sqrt(rand(stamp("../copySegments","seg",0)*stamp("../finalcopy","ccy",1))*prim("../attribpromote2",0,"distP",0)/point("../carve1",0,"dist",0))')
		pea2.setFirstInput(car)

		
		ski2 = self.fusionar(car,pea2)
		
		#b = buildings()
		#aux = b.buildings(self.geo, ski2, buildingName)

		copy3 = self.geo.createNode('copy')
		copy3.setName('copySegments')
		copy3.parm('ncy').setExpression('npoints("../point1")/3') #as many copies as number of points
		copy3.parm('stamp').set(1)
		copy3.parm('param1').set('seg') 
		copy3.parm('val1').setExpression("$CY") #the value is the copy number
		copy3.setFirstInput(ski2)

		copy2 = self.geo.createNode('copy')
		copy2.setName('finalcopy')
		copy2.parm('ncy').setExpression('nprims("../entryLots")') #number of copies
		copy2.parm('stamp').set(1)
		copy2.parm('param1').set('ccy') 
		copy2.parm('val1').setExpression("$CY") #the value is the copy number
		copy2.setFirstInput(copy3)

		controlVariables.numPrims = copy2.parm('ncy').eval()
		print(controlVariables.numPrims)

		self.last = copy2
	
	def dividirParceles(self,mea3):
		res3 = self.geo.createNode('resample')
		res3.setFirstInput(mea3)
		res3.parm('dolength').set(0)
		res3.parm('dosegs').set(1)
		self.resampleNode = res3
		self.__assignResampleExpr()

		#divide in little segments
		res2 = self.geo.createNode('resample')
		res2.setFirstInput(res3)
		res2.parm('edge').set(1)
		res2.parm('dolength').set(0)
		res2.parm('dosegs').set(1)
		res2.parm('segs').set(3)
		
		poi = self.roundAngles(res2)
		
		return poi

	def __assignResampleExpr(self):
		#original expr: '8 * prim("../measurePerim",0,"perim",0) / detail("../avgperim","avgperim",0)'
		primExpr = 'prim("../measurePerim",0,"perim",0)'
		finalExpr = '8 * ' + primExpr + ' / ' + self.detailExpr
		self.resampleNode.parm('segs').setExpression(finalExpr)

	def parmSet(self, parmName, parmValue):
		if (parmName == 'detailExpr'):
			self.detailExpr = parmValue
			self.__assignResampleExpr()
		else:
			#ERROR
			pass
		
	def roundAngles(self,res2):
		cir = self.geo.createNode('circle')
		cir.parm('type').set(2) #NURBS curve
		cir.parm('divs').setExpression('npoints("../resample2")')
	
		#force the circle to have the shape of resample
		poi = self.geo.createNode('point')
		poi.setFirstInput(cir)
		poi.setInput(1,res2)
		poi.parm('tx').setExpression('$TX2') #it takes the position of the resample
		poi.parm('ty').setExpression('$TY2')
		poi.parm('tz').setExpression('$TZ2')
	
		return poi	
		
	def crearPrimitivaInterior(self,mea3):
		#the inner primitive
		poli3 = self.geo.createNode('polyextrude')
		poli3.setName('buildingSize')
		poli3.parm('lsx').set(controlVariables.lotsWidth) #scale
		poli3.parm('lsy').set(controlVariables.lotsWidth)
		poli3.parm('outputside').set(0) #just output front
		
		ray = self.geo.createNode('ray')
		ray.setInput(1,poli3)
		ray.parm('entity').set(1) #points
		ray.parm('method').set(0) #minimum distance
		ray.parm('putdist').set(1) #point intersection distance

		#the new normal
		poi2 = self.geo.createNode('point')
		poi2.setInput(1,ray)
		poi2.parm('donml').set(1) #add normal
		poi2.parm('nx').setExpression("$TX2-$TX") #the current point of the second input minus
		poi2.parm('ny').setExpression("$TY2-$TY")  #the current point of the first input
		poi2.parm('nz').setExpression("$TZ2-$TZ")
	
		poi = self.dividirParceles(mea3)
		poli3.setFirstInput(poi)
		ray.setFirstInput(poi)
		poi2.setFirstInput(poi)
	
		#the distance between corresponding points of the inner and outer primitives
		#is an attribute of the inner primitive and we want to transfer it to the outer one
		att = self.geo.createNode('attribtransfer')
		att.setFirstInput(poi2)
		att.setInput(1,ray)
		att.parm('srcgrouptype').set(1) #points
		att.parm('dstgrouptype').set(1) #points
		att.parm('pointattriblist').set('dist')
	
		attp = self.geo.createNode('attribpromote')
		attp.setName('distPAttriPromote')
		attp.setFirstInput(att)
		attp.parm('inname').set('dist')
		attp.parm('outclass').set(1)
		attp.parm('method').set(1) #minimum
		attp.parm('useoutname').set(1)
		attp.parm('outname').set('distP')
		attp.parm('deletein').set(0)
	
		return attp
	
	def fusionar(self,car,pea2):
		#merge the inner and the outer primitives
		mer2 = self.geo.createNode('merge')
		mer2.setFirstInput(car)
		mer2.setInput(1,pea2)
		
		#join the segments
		ski2 = self.geo.createNode('skin')
		ski2.setFirstInput(mer2)

		return ski2