import hou
from controlVariables import *

def makeHDA(container):
	#First, collapse the container into a subnet
	citySubnet = container.collapseIntoSubnet(container.children(), subnet_name = 'citySubnet')
	#now, create the HDA from the subnet
	cityHDANode = citySubnet.createDigitalAsset(name = 'cityHDA',
                                description = 'city HDA',
                                ignore_external_references = True) 
	cityHDANode.moveToGoodPosition()
	# Get the nodeType definition of the new city asset. 
	cityDefinition = cityHDANode.type().definition()


	#======================================
	# Create Parameter Interface
	#======================================
	#For each param, it seems that the steps are:
	# 1 - create a parm template. Do not forget to provide the number of components... and the dafault values.
	# 2 - add that parm template to the HDA definition
	citySize = hou.FloatParmTemplate(name='citySize', label='City scale', 
		num_components=3, 
		default_value=(1,1,1)
		)
	cityDefinition.addParmTuple(citySize, in_folder=("Params",), create_missing_folders=True)
	
	
	#nPrims = hou.node('obj/geo1/citySubnet/finalcopy').parm('ncy').eval()
	#print nPrims
	visibleLotsMin = hou.IntParmTemplate(name='visibleLotsMin', label='Visible lots min',
		num_components=1,
		default_value=(0,)
		)
	cityDefinition.addParmTuple(visibleLotsMin, in_folder=("Params",), create_missing_folders=True)
	
	visibleLotsMax = hou.IntParmTemplate(name='visibleLotsMax', label='Visible lots max',
		num_components=1,
		default_value=(0,),
		#min = hou.node('obj/geo1/citySubnet').parm('visibleLotsMin').eval(),
		#min_is_strict=True,
		max = controlVariables.numPrims
		)
	cityDefinition.addParmTuple(visibleLotsMax, in_folder=("Params",), create_missing_folders=True)

	
	if controlVariables.loadMap==None:
		lotSize = hou.FloatParmTemplate(name='lotSize', label='Lot width', 
			num_components=2, 
			default_value=(1.1,1.1),
			)
		cityDefinition.addParmTuple(lotSize, in_folder=("Params",), create_missing_folders=True)
		
		blockSize = hou.FloatParmTemplate(name='blockSize', label='Block scale', 
			num_components=2, 
			default_value=(float(controlVariables.blockSizex),float(controlVariables.blockSizey))
			)
		cityDefinition.addParmTuple(blockSize, in_folder=("Params",), create_missing_folders=True)

		if controlVariables.streetsAngle!='random':
			streetRotation = hou.FloatParmTemplate(name='streetRotation', label='Street Rotation',
				num_components=1,
				default_value=(float(controlVariables.streetsAngle),),
				min = 0,
				min_is_strict = True,
				max = 360,
				max_is_strict = True
				)
			cityDefinition.addParmTuple(streetRotation, in_folder=("Params",), create_missing_folders=True)
		#else:
			#streetRotation = hou.FloatParmTemplate(name='streetRotation', label='Street Rotation',
			#	num_components=1,
			#	default_value=(hou.hscript('360*(rand(stamp("../copy1","cy",0)))'))
			#	)
			#cityDefinition.addParmTuple(streetRotation, in_folder=("Params",), create_missing_folders=True)
			#cityDefinition.addParmTuple(streetRotation, in_folder=("Params",), create_missing_folders=True)
		
	buildingSize = hou.FloatParmTemplate(name='buildingSize', label='Inner lot scale',
			num_components=2, 
			default_value=(float(controlVariables.lotsWidth),float(controlVariables.lotsWidth))
			)
	cityDefinition.addParmTuple(buildingSize, in_folder=("Params",), create_missing_folders=True)
	

	#=========================================
	# Link parameters with corresponding nodes
	#=========================================
	# Linking is simple: just add an expression referring to the parent channel
	scaleNodes = findSubNodesWith(cityHDANode,'finalScale')
	for scale in scaleNodes:
		scale.parm("sx").setExpression('ch("../citySizex")')
		scale.parm("sy").setExpression('ch("../citySizey")')
		scale.parm("sz").setExpression('ch("../citySizez")')
			
	visible = findSubNode(cityHDANode,'visibles')
	visible.parm("rangestart").setExpression('ch("../visibleLotsMin")')
	visible.parm("rangeend").setExpression('ch("../visibleLotsMax")')
	#visible.parm("rangeend").setExpression('if ch("../visibleLotsMax") <  ch("../visibleLotsMin"):\n   $N\nelse:\n   ch("../visibleLotsMax")')
	
	if controlVariables.loadMap==None:
		scaleLots = findSubNode(cityHDANode,'scaleLots')
		scaleLots.parm("sx").setExpression('ch("../lotSizex")')
		scaleLots.parm("sy").setExpression('ch("../lotSizey")')
		#scaleLots.parm("sz").setExpression('ch("../lotSizez")')
		
		blockSize = findSubNode(cityHDANode,'blockSize')
		blockSize.parm("lsx").setExpression('ch("../blockSizex")')
		blockSize.parm("lsy").setExpression('ch("../blockSizey")')
		
		if controlVariables.streetsAngle!='random':
			rotation = findSubNode(cityHDANode,'rotation')
			rotation.parm("rz").setExpression('ch("../streetRotation")')
		
	buildingSize = findSubNode(cityHDANode,'buildingSize')
	buildingSize.parm("lsx").setExpression('ch("../buildingSizex")')
	buildingSize.parm("lsy").setExpression('ch("../buildingSizey")')

	
def findSubNode(searchSpace, nodeName):
	theNode = None
	for n in searchSpace.children():
		if (n.name() == nodeName):
			theNode = n
	return theNode

def findSubNodesWith(searchSpace, nodeName):
	theNodes = []
	for n in searchSpace.children():
		if (n.name().find(nodeName)>=0):
			theNodes.append(n)
	return theNodes	
