import sys
import hou

from controlVariables import *
from regularPattern import *

class radial(regularPattern):

	def createSurface(self,type,districts=None):

		sur = controlVariables.sur
		glo = sur.createNode('global')

		fltovec = sur.createNode('floattovec')
		if type == 'surface':
			fltovec.setFirstInput(glo,11)
			fltovec.setInput(1,glo,12) #our global variable is now a vector
		elif type == 'filter':
			fltovec.setFirstInput(glo,22)
			fltovec.setInput(1,glo,23) #our global variable is now a vector

		#the point in the center
		cons = sur.createNode('constant')
		cons.parm('consttype').set('vector')
		cons.parm('vectordef1').set(controlVariables.radialCx)
		cons.parm('vectordef2').set(controlVariables.radialCx)
		cons.parm('vectordef3').set('0')

		#compute the distance between the center point and the current point
		dist = sur.createNode('distance')
		dist.setFirstInput(fltovec)
		dist.setInput(1,cons)
		
		#the radial frequence
		mulconst = sur.createNode('mulconst')
		mulconst.setFirstInput(dist)
		mulconst.parm('mulconst').set(15)
		#the radial stripes
		str = sur.createNode('stripes')
		str.setFirstInput(mulconst)
		str.parm('width').set(0.05)
		str.parm('blur').set(1)
		
		vectofl = sur.createNode('vectofloat')
		vectofl.setFirstInput(cons)
		
		sub = sur.createNode('subtract')
		sub2 = sur.createNode('subtract')
		if type == 'surface':
			sub.setFirstInput(glo,11)
			sub2.setInput(1,glo,12)
		elif type == 'filter':
			sub.setFirstInput(glo,22)
			sub2.setInput(1,glo,23)


		sub.setInput(1,vectofl)
		sub2.setInput(1,vectofl,1)
				
		div = sur.createNode('divide')
		div.setFirstInput(sub)
		div.setInput(1,sub2)
		
		trig = sur.createNode('trig')
		trig.setFirstInput(div)
		trig.parm('func').set('vop_atan') #arctangent
		
		#the angular stripes
		str2 = sur.createNode('stripes')
		str2.setFirstInput(trig)
		str2.parm('width').set(0.05)
		str2.parm('freq').set(3)
		str2.parm('blur').set(1)
		
		add = sur.createNode('add')
		add.setFirstInput(str)
		add.setInput(1,str2)

		if districts == None:
			if type == 'surface':
				fltovec2 = sur.createNode('floattovec')
				fltovec2.setFirstInput(add)
				fltovec2.setInput(1,add)
				fltovec2.setInput(2,add)
				
				#wire it to he output node
				out = hou.node('/vex/CityPattern/output1')
				out.setFirstInput(fltovec2)
									
			elif type == 'filter':
				out = hou.node('/vex/myPattern/output1')
				out.setFirstInput(add)
				out.setInput(1,add)
				out.setInput(2,add)
		else:
			if type == 'surface':
				fltovec2 = sur.createNode('floattovec')
				fltovec2.setFirstInput(add)
				fltovec2.setInput(1,add)
				fltovec2.setInput(2,add)
				
				#wire it to he output node
				out = districts
				out.setInput(1,fltovec2)
									
			elif type == 'filter':
				out = districts
				out.setInput(1,add)
				out.setInput(2,add)
				out.setInput(3,add)

		for n in sur.children():
			   n.moveToGoodPosition()