# ============================================================
# =====       Simple Building                            =====
# ============================================================
#
# This class produces a simple textured building. The idea is 
# to replace it by a building from our buildingEngine.

print "  SimpleBuilding module imported"

import hou

from controlVariables import *

class SimpleBuilding:
	def __init__(self):
		pass

	def createGeometry(self,texturePath):
		
		geoContainer = hou.node('/obj').createNode('geo')
		for n in geoContainer.children(): 
			n.destroy()
		geoContainer.setName('simpleBuilding')
		geoContainer.setDisplayFlag(False)

		theBox = geoContainer.createNode('box')
		theBox.parm('type').set('polymesh')
		theBox.parm('sizex').set(1)
		theBox.parm('sizey').set(1)
		theBox.parm('sizez').set(0.15)	  
		theBox.parm('divrate1').set(4+1)
		theBox.parm('divrate2').set(5+1)
		theBox.parm('divrate3').set(2+1)

		# ROOF ===================================
		dele1 = geoContainer.createNode('delete')
		dele1.parm('negate').set('keep')
		dele1.parm('groupop').set('filter')
		dele1.parm('filter').setExpression("$TY==$YMAX")
		dele1.setFirstInput(theBox)
		uv1 = geoContainer.createNode('uvquickshade')
		uv1.parm('texture').set(texturePath+'roof185.jpg')
		uv1.parm('texture_axis').set(1)
		uv1.parm('texture_su').setExpression("ch('../box1/divrate1')-1")#(4)
		uv1.parm('texture_sv').setExpression("ch('../box1/divrate3')-1")#(2)
		uv1.setFirstInput(dele1)

		# Sides ==================================
		dele2 = geoContainer.createNode('delete')
		dele2.parm('negate').set('keep')
		dele2.parm('groupop').set('filter')
		dele2.parm('filter').setExpression("$TX==$XMAX||$TX==$XMIN")
		dele2.setFirstInput(theBox)
		uv2 = geoContainer.createNode('uvquickshade')
		uv2.parm('texture').set(texturePath+'brickwalls.jpg')
		uv2.parm('texture_axis').set(0)
		uv2.parm('texture_su').setExpression("ch('../box1/divrate3')-1")#(2)
		uv2.parm('texture_sv').setExpression("ch('../box1/divrate2')-1")#(5)
		uv2.setFirstInput(dele2)

		# Facade =================================
		dele3 = geoContainer.createNode('delete')
		dele3.parm('negate').set('keep')
		dele3.parm('groupop').set('filter')
		dele3.parm('filter').setExpression("$TZ==$ZMAX||$TZ==$ZMIN")
		dele3.setFirstInput(theBox)
		# Facade: windows ========================
		dele4 = geoContainer.createNode('delete')
		dele4.parm('negate').set('dele')
		#dele4.parm('groupop').set('pattern')
		#dele4.parm('pattern').set(0)
		dele4.setExpressionLanguage(hou.exprLanguage.Hscript)
		dele4.parm('group').setExpression("floor(rand(1)*(ch('../box1/divrate1')-1))")#`floor(rand(ch("../seed"))*nprims("../dele3"))`)
		dele4.setFirstInput(dele3)
		uv3 = geoContainer.createNode('uvquickshade')
		uv3.parm('texture').set(texturePath+'RegWindow3.jpg')
		uv3.parm('texture_axis').set(2)
		uv3.parm('texture_su').setExpression("ch('../box1/divrate1')-1")#(4)
		uv3.parm('texture_sv').setExpression("ch('../box1/divrate2')-1")#(5)
		uv3.setFirstInput(dele4)
		# Facade: door ===========================
		dele5 = geoContainer.createNode('delete')
		dele5.parm('negate').set('keep')
		#dele5.parm('groupop').set('pattern')
		#dele5.parm('pattern').set('0')
		dele5.setExpressionLanguage(hou.exprLanguage.Hscript)
		dele5.parm('group').setExpression("floor(rand(1)*(ch('../box1/divrate1')-1))")#`floor(rand(ch("../seed"))*nprims("../dele3"))`)
		dele5.setFirstInput(dele3)
		uv4 = geoContainer.createNode('uvquickshade')
		uv4.parm('texture').set(texturePath+'door.jpg')
		uv4.parm('texture_axis').set(2)
		uv4.parm('texture_su').set(1)
		uv4.parm('texture_sv').set(1)
		uv4.setFirstInput(dele5)

		#Merge them all ==========================
		merge = geoContainer.createNode('object_merge')
		merge.parm('numobj').set(5)
		merge.parm("objpath1").set("../uvquickshade1")
		merge.parm("objpath2").set("../uvquickshade2")
		merge.parm("objpath3").set("../uvquickshade3")
		merge.parm("objpath4").set("../uvquickshade4")

		trans = geoContainer.createNode('xform')
		trans.parm('rx').set(90)
		#trans.parm('tz').set(-0.125)
		trans.setFirstInput(merge)

		trans.setDisplayFlag(True)

		for n in geoContainer.children(): 
			n.moveToGoodPosition()

		return geoContainer
	
	def createBuilding(self, HDAName, texturePath):
		container = self.createGeometry(texturePath)
		self.createHDA(HDAName,container)

	def createHDA(self, HDAName, container):
		#First, collapse the container (the geo node called box_object1) into a subnet
		buildingSubnet = container.collapseIntoSubnet(container.children(), subnet_name = 'buildingSubnet')
		#now, create the HDA from the subnet
		buildingHDANode = buildingSubnet.createDigitalAsset(name = HDAName,
					#hda_file_name = filename,
					description = 'simple building HDA',
					ignore_external_references = True) 
		buildingHDANode.moveToGoodPosition()
		# Get the nodeType definition of the new building asset. 
		buildingDefinition = buildingHDANode.type().definition()

		#======================================
		# Create Parameter Interface
		#======================================
		#For each param, it seems that the steps are:
		# 1 - create a parm template. Do not forget to provide the numbar of components... and the dafault values.
		# 2 - add that parm template to the HDA definition
		size_template = hou.IntParmTemplate(name='boxHeight', label='Floors', 
			num_components=1, 
			default_value=(4,)
			)
		buildingDefinition.addParmTuple(size_template, in_folder=("Params",), create_missing_folders=True)

		size_template = hou.IntParmTemplate(name='boxWidth', label='Columns', 
			num_components=1, 
			default_value=(5,)
			)
		buildingDefinition.addParmTuple(size_template, in_folder=("Params",), create_missing_folders=True)

		seedParm_template = hou.FloatParmTemplate(name='doorSeed', label='Door Seed', 
			num_components=1, 
			default_value=(1,)
			)
		buildingDefinition.addParmTuple(seedParm_template, in_folder=("Params",), create_missing_folders=True)

		#=========================================
		# Link parameters with corresponding nodes
		#=========================================
		# Linking is simple: just add an expression referring to the parent channel
		theBox = self.findSubNode(buildingHDANode,'box1')
		#theBox.parm("sizex").setExpression('ch("../boxHeight")')
		#theBox.parm("sizey").setExpression('ch("../boxWidth")')
		#theBox.parm("sizez").setExpression('ch("../boxSizez")')
		theBox.parm("divrate1").setExpression('ch("../boxWidth")+1')
		theBox.parm("divrate2").setExpression('ch("../boxHeight")+1')
		#theBox.parm("divrate3").setExpression('ch("../boxSizez")+1')

		deleA = self.findSubNode(buildingHDANode,'delete4')
		deleA.parm('group').setExpression("floor(rand(ch('../doorSeed'))*(ch('../box1/divrate1')-1))")
		deleB = self.findSubNode(buildingHDANode,'delete5')
		deleB.parm('group').setExpression("floor(rand(ch('../doorSeed'))*(ch('../box1/divrate1')-1))")

		asset = hou.node('/obj/simpleBuilding')
		asset.parm('rx').set(-90)
		#buildingHDANode.parm('rx').set(-90)

	def findSubNode(self, searchSpace, nodeName):
		theNode = None
		for n in searchSpace.children():
			if (n.name() == nodeName):
				theNode = n
		return theNode


def buildBuildingHDA(texturePath):
	build = SimpleBuilding().createBuilding('buildingHDA',texturePath)
