import hou

from controlVariables import *
import uENode
import Streets
import errorMsg

print "  stdNetworkNodes module imported"

# ===========================================================
#                Standard Null Nodes
# ===========================================================
class Null_uENode(uENode.urbanEngineNode):
	def __init__(self):
		uENode.urbanEngineNode.__init__(self)
	def getNodeName(self):
		return 'NullNode'
	def construct(self):
		#Simply do nothing.
		pass

# ===========================================================
#                DicedMinorRoads
# ===========================================================
class DicedMinorRoads(uENode.urbanEngineNode):
	def __init__(self):
		uENode.urbanEngineNode.__init__(self)
		self.box = None
		self.detailExpr = ''
	def getNodeName(self):
		return 'DicedMinorRoads'
	def construct(self):
		s = Streets.streets()
		newStreets, self.box = s.streets(self.first)
		self.__build_divrateExpr()
		self.last = newStreets
	def parmSet(self, parmName, parmValue):
		if (parmName == 'detailExpr'):
			self.detailExpr = parmValue
			self.__build_divrateExpr()
		else:
			#ERROR
			pass
	def __build_divrateExpr(self):
		#original expr: '( (prim("../measureLot",0,"areaLot",0)/detail("../avgarea","avgarea",0)) *3)+2'
		#               '( (prim("../measureLot",0,"areaLot",0)/detail("../avgarea","avgarea",0)) *2)+2'
		measureLotExpr = 'prim("../measureLot",0,"areaLot",0)'
		finalExpr = '(' + measureLotExpr + ' / '  + self.detailExpr + ')'
		self.box.parm('divrate1').setExpression('(' + finalExpr + '*3)+2')
		self.box.parm('divrate2').setExpression('(' + finalExpr + '*2)+2')

# ===========================================================
#                Blocks
# ===========================================================
class Blocks(uENode.urbanEngineNode):
	def __init__(self):
		uENode.urbanEngineNode.__init__(self)
		self.areaExpr = ''
		self.detailExpr = ''
	def getNodeName(self):
		return 'Blocks'
	def construct(self):
		#delete too small primitives
		del5 = self.geo.createNode('delete')
		del5.setName('del')
		del5.parm('groupop').set(2) #delete by expression
		del5.setFirstInput(self.first)
		self.last = del5
		self.__assignFilterExpr()
	def __assignFilterExpr(self):
		#original expr: '$AREA < 0.7 * detail("../areaMeasurer","totsize",0)'
		finalExpr = self.areaExpr + ' < ' + controlVariables.minimumPercentualSize + ' * ' + self.detailExpr
		self.last.parm('filter').setExpression(finalExpr)
	def parmSet(self, parmName, parmValue):
		if (parmName == 'areaExpr'):
			self.areaExpr = parmValue
			self.__assignFilterExpr()
		elif (parmName == 'detailExpr'):
			self.detailExpr = parmValue
			self.__assignFilterExpr()
		else:
			#ERROR
			pass

# ===========================================================
#                FinalAppearance
# ===========================================================
class FinalAppearance(uENode.urbanEngineNode):
	def __init__(self):
		uENode.urbanEngineNode.__init__(self)
		self.i = 0
	def getNodeName(self):
		return 'FinalAppearance'

	def __scale(self, tra, name):
		#scale the whole terrain
		scale2 = self.geo.createNode('xform')
		scale2.setName(name)
		scale2.parm('sx').set(1)
		scale2.parm('sy').set(1)
		scale2.parm('sz').set(controlVariables.buildingScale)#should only have a visible effect on buildings...
		scale2.setFirstInput(tra.getLast())
		return scale2

	def construct(self):
		self.first.setDisplayFlag(True)

	def setFirstInput(self, node_to_become_input, output_index=0):
		scale = self.__scale(node_to_become_input,'finalScale'+str(self.i))
		self.i += 1
		self.first.setFirstInput(scale, output_index)

	def setNextInput(self, node_to_become_input, output_index=0):
		scale = self.__scale(node_to_become_input,'finalScale'+str(self.i))
		self.i += 1
		self.first.setNextInput(scale, output_index)

	def setInput(self, input_index, node_to_become_input, output_index=0):
		scale = self.__scale(node_to_become_input,'finalScale'+str(self.i))
		self.i += 1
		self.first.setInput(input_index, scale, output_index)


# ===========================================================
#                Floor
# ===========================================================
class Floor(uENode.urbanEngineNode):
	def __init__(self):
		uENode.urbanEngineNode.__init__(self)
		self.matchsize = None
	def getNodeName(self):
		return 'Floor'

	def construct(self):
		grid = self.geo.createNode('grid')
		grid.parm('orient').set(0) #xy pane
		grid.parm('sizex').set(1)
		grid.parm('sizex').set(1)

		shade = self.geo.createNode('uvquickshade')
		shade.setFirstInput(grid)
		shade.parm('texture').set(controlVariables.floor)
		shade.parm('texture_axis').set(2) #z axis

		temp = self.geo.createNode('null')
		temp.setName('floorSizeRef')
		
		self.matchsize = self.geo.createNode('matchsize')
		self.matchsize.setFirstInput(shade)
		self.matchsize.setInput(1,temp)

		xform = self.geo.createNode('xform')
		xform.setFirstInput(self.matchsize)
		xform.parm('tz').set(-0.04)

		temp.setFirstInput(self.first)
		self.last = xform

# ===========================================================
#                ConstructionFilter
# ===========================================================
class ConstructionFilter(uENode.urbanEngineNode):
	def __init__(self):
		uENode.urbanEngineNode.__init__(self)
	def getNodeName(self):
		return 'ConstructionFilter'
	def construct(self):
		del7 = self.geo.createNode('delete')
		del7.parm('negate').set(1) #delete non-selected
		del7.parm('groupop').set(1) #delete by range
		del7.parm('select1').set(1)
		del7.parm('select2').set(1)
		del7.parm('rangestart').set('0')
		#del7.parm('rangeend').setExpression('0')
		del7.parm('rangeend').setExpression('$N')
		del7.setFirstInput(self.first)		
		self.last = del7

# ===========================================================
#                Stats
# ===========================================================
class Stats(uENode.urbanEngineNode):
	def __init__(self):
		uENode.urbanEngineNode.__init__(self)
		self.avgStatsNode = None
		self.myName = ''
	def getNodeName(self):
		return 'Stats'
	def construct(self):
		mea2 = self.geo.createNode('measure')
		mea2.setName('measure_Stats')
		mea2.parm('override').set(1) #override name
		mea2.parm('type').set(0)  #compute: 0=perimeters, 1=areas
		mea2.parm('attribname').set('measure_Value')
		mea2.setFirstInput(self.first)

		attpr2 = self.geo.createNode('attribpromote')
		attpr2.setFirstInput(mea2)
		attpr2.setName('avgStats')
		attpr2.parm('inname').set('measure_Value') #original name
		attpr2.parm('inclass').set(1) #original class: primitive
		attpr2.parm('outclass').set(0) #new class: detail
		attpr2.parm('method').set(2) #promotion method: average
		attpr2.parm('useoutname').set(1) #change new name
		attpr2.parm('outname').set('avgValue')	
		attpr2.parm('deletein').set(0) #don't delete the original
		self.avgStatsNode = attpr2

		self.last = mea2
	def setName(self, aName):
		self.myName = aName
		self.first.setName(aName + '_inputMerge')
		self.last.setName(aName + '_Stats')
		self.last.parm('attribname').set(aName + '_Value')
		self.avgStatsNode.setName(aName + '_avgStats')
		self.avgStatsNode.parm('inname').set(aName + '_Value')
		self.avgStatsNode.parm('outname').set(aName + '_avgValue')
	def name(self):
		return self.myName
	def parmSet(self, parmName, parmValue):
		# OK, this should be fixed to have more flexibility...
		if (parmName == 'type'):
			self.last.parm('type').set(parmValue)
		else:
			#errorMsg.parmUndef(parmName)
			pass
	def parmEval(self, parmName):
		if (parmName == 'path'):
			expr = 'detail("' + self.avgStatsNode.path() + '","' + self.myName + '_avgValue",0)'
			return hou.hscriptExpression(expr)
		else:
			#errorMsg.parmUndef(parmName)
			pass
