import sys
import hou

from densityRadial import *
from densityImg import *
from controlVariables import *
from regularPattern import *

class voronoise(regularPattern):

	def createSurface(self,type,districts=None):

		sur = controlVariables.sur
		glo = sur.createNode('global')
	
		vor = sur.createNode('voronoise')

		fltovec = sur.createNode('floattovec')
		if type == 'surface':
				fltovec.setFirstInput(glo,11)
				fltovec.setInput(1,glo,12) #our global variable is now a vector
		elif type == 'filter':
				fltovec.setFirstInput(glo,22)
				fltovec.setInput(1,glo,23) #our global variable is now a vector

		#the point in the center
		cons = sur.createNode('constant')
		cons.parm('consttype').set('vector')
		cons.parm('vectordef1').set(controlVariables.voronoiCx)
		cons.parm('vectordef2').set(controlVariables.voronoiCy)
		cons.parm('vectordef3').set('0')


		#multiply the frequency by a constant
		mult = sur.createNode('mulconst')
		mult.parm('mulconst').set(controlVariables.voronoiFrequency)
		vor.setInput(1,mult)
	
		#density control
		if controlVariables.voronoiDensity == 'centered':
			d = densityRadial()
		else:
			d = densityImg()
		mult.setFirstInput(d.density(type,glo,fltovec))

		#create a random node, link it to voronoi noise
		ran = sur.createNode('random')
		ran.parm("signature").set("vv")
		ran.setFirstInput(vor,2)

		sub = sur.createNode('subtract')
		sub.setFirstInput(fltovec)
		sub.setInput(1,cons)
		vor.setFirstInput(sub)

		if districts == None:
			if type == 'surface':
				#wire it to he output node
				out = hou.node('/vex/CityPattern/output1')
				out.setFirstInput(ran)
			elif type == 'filter':
				vectofl = sur.createNode('vectofloat')
				vectofl.setFirstInput(ran)
				out = hou.node('/vex/myPattern/output1')
				out.setFirstInput(vectofl)
				out.setInput(1,vectofl,1)
				out.setInput(2,vectofl,2)
		else:
			if type == 'surface':
				#wire it to he output node
				out = districts
				out.setInput(1,ran)
			elif type == 'filter':
				vectofl = sur.createNode('vectofloat')
				vectofl.setFirstInput(ran)
				out = districts
				out.setInput(1,vectofl)
				out.setInput(2,vectofl,1)
				out.setInput(3,vectofl,2)
		


		for n in sur.children():
			   n.moveToGoodPosition()